/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <cassert>
#include <cstddef>
#include <vector>
#include <utility>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/Operator.hpp"
#include "aidge/utils/Types.h"
#include "aidge/utils/ErrorHandling.hpp"

// constexpr Aidge::Operator::Operator(const char* type)
//     : mType(type)
// {
// 	// ctor
// }

Aidge::Operator::~Operator() = default;

///////////////////////////////////////////////////////
//        IMPLEMENTATION
///////////////////////////////////////////////////////

// std::vector<std::pair<std::size_t, std::vector<Aidge::DimSize_t>>> Aidge::Operator::computeReceptiveField(
//         const std::size_t firstIdx, const std::vector<Aidge::DimSize_t>& outputDims, const Aidge::IOIndex_t outputIdx) const
// {
//     static_cast<void>(outputIdx);
//     if (outputIdx >= nbOutputs()) {
//         AIDGE_THROW_OR_ABORT(std::runtime_error, "Operator output index out of range.");
//     }
//     if (nbInputs() != nbDataInputs()) {
//         AIDGE_THROW_OR_ABORT(std::runtime_error, "Operator has attributes. Must be handled in an overrided function.");
//     }
//     if (!outputDimsForwarded() || getOutput(0)->nbDims() != outputDims.size()) {
//         AIDGE_THROW_OR_ABORT(std::runtime_error, "Given outputDim out of range or output dim not forwarded yet.");
//     }
//     const auto outputIdxDims = getOutput(0)->getCoord(firstIdx);
//     for (DimIdx_t i = 0; i < outputDims.size(); ++i) {
//         if (((outputDims[i] + outputIdxDims[i]) > getOutput(0)->dims()[i]) || (outputDims[i] == 0)) {
//             AIDGE_THROW_OR_ABORT(std::runtime_error, "Given outputDim out of range for dimension %lu (%lu + %lu)", static_cast<std::size_t>(i), outputIdxDims[i], outputDims[i]);
//         }
//     }
//     // return the same Tensor description as given in function parameter for each data input
//     return std::vector<std::pair<std::size_t, std::vector<Aidge::DimSize_t>>>(nbDataInputs(),std::pair<std::size_t, std::vector<Aidge::DimSize_t>>(firstIdx, outputDims));
// }

Aidge::NbElts_t Aidge::Operator::getNbRequiredData(const Aidge::IOIndex_t inputIdx) const {
    return mImpl->getNbRequiredData(inputIdx);
}

Aidge::NbElts_t Aidge::Operator::getNbConsumedData(Aidge::IOIndex_t inputIdx) const {
    return mImpl->getNbConsumedData(inputIdx);
}

Aidge::NbElts_t Aidge::Operator::getNbProducedData(Aidge::IOIndex_t outputIdx) const {
    return mImpl->getNbProducedData(outputIdx);
}
void Aidge::Operator::updateConsummerProducer(){
    mImpl->updateConsummerProducer();
}

void Aidge::Operator::runHooks() const {
    for (auto& hook : mHooks) {
        hook.second->call();
    }
}
void Aidge::Operator::forward() {
    mImpl->forward();
    runHooks();
}

void Aidge::Operator::backward() { mImpl->backward(); }
